/******************************************************************************\
|*                                                                            *|
|*                           FaceCell Matcher 1.1                             *|
|*                                                                            *|
|* FccMatcher.h                                                               *|
|* Header file for FccMatcher module                                          *|
|*                                                                            *|
|* Copyright (C) 2006-2007 Neurotechnologija                                  *|
|*                                                                            *|
\******************************************************************************/

#ifndef FCC_MATCHER_H_INCLUDED
#define FCC_MATCHER_H_INCLUDED

#include "FccMatcherParams.h"

#ifdef N_CPP
extern "C"
{
#endif

/// Handle to the matcher object.
N_DECLARE_HANDLE(HFccMatcher);

#ifndef N_NO_ANSI_FUNC
NResult N_API FccmGetInfoA(NLibraryInfoA * pValue);
#endif
#ifndef N_NO_UNICODE
NResult N_API FccmGetInfoW(NLibraryInfoW * pValue);
#endif
#define FccmGetInfo N_FUNC_AW(FccmGetInfo)

/**
 * \brief  Checks whether FccMatcher library is registered.
 * \return  #NTrue if library is registered, #NFalse otherwise.
 */
NBool N_API FccmIsRegistered(void);

/**
 * \brief  Creates matcher object.
 * \param[out] pHMatcher  Pointer to #HFccMatcher that receives handle to
 *                        the created matcher object.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p pHMatcher is #NULL</td></tr>
 *            <tr><td>#N_E_OUT_OF_MEMORY</td>
 *                <td>There was not enough memory.</td></tr>
 *          </table>
 * \remark  Created object must be deleted using #FccmFree function.
 * \sa  HFccMatcher | FccmFree
 *
 * Creates matcher object and sets all parameters to default values.
 */
NResult N_API FccmCreate(HFccMatcher *pHMatcher);
/**
 * \brief  Resets matching object.
 * \param[in]  hMatcher  Handle to the matcher object.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hMatcher is #NULL</td></tr>
 *          </table>
 * \sa  HFccMatcher
 *
 * Sets default values for all parameters of the specified matcher.
 */
NResult N_API FccmReset(HFccMatcher hMatcher);
/**
 * \brief  Deletes matcher object.
 * \param[in]  hMatcher  Handle to the matcher object.
 * \remark  If \p hMatcher is #NULL, does nothing.
 * \sa  HFccMatcher | FccmCreate
 *
 * Deletes matcher object. After the object is deleted the specified handle
 * is no longer valid.
 */
void N_API FccmFree(HFccMatcher hMatcher);

#ifdef N_DOCUMENTATION
/**
 * \brief  Retrieves value of the parameter.
 * \param[in]  hMatcher     Handle to the matcher object.
 * \param[in]  parameterId  Identifier of the parameter to retrieve.
 * \param[out] pValue       Pointer to a variable that receives parameter
 *                          value.
 * \return  If the function succeeds and \p parameterId specifies
 *          #N_TYPE_STRING type parameter and \p pValue is #NULL, the return
 *          value is length of the string (not including NULL-terminator)
 *          \p pValue should point to.
 * \return  If the function succeeds and \p pValue is not #NULL, the return
 *          value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td rowspan="2">#N_E_ARGUMENT_NULL</td>
 *                <td>\p parameterId specifies a non-static parameter and
 *                \p hMatcher is #NULL</td></tr>
 *            <tr><td>\p parameterId specifies a non-#N_TYPE_STRING type
 *                parameter and \p pValue is #NULL.</td></tr>
 *            <tr><td>#N_E_PARAMETER</td>
 *                <td>\p parameterId is invalid.</td></tr>
 *          </table>
 * \remark  The following values can be used for \p parameterId:
 *          - #FCCMP_MATCHING_THRESHOLD
 *          - #FCCMP_NAME
 *          - #FCCMP_VERSION_HIGH
 *          - #FCCMP_VERSION_LOW
 *          - #FCCMP_COPYRIGHT
 *          To learn the type of parameter pass a value obtained with
 *          #NParameterMakeId macro using #N_PC_TYPE_ID code, the parameter
 *          identifier via \p parameterId and pointer to #NInt that will
 *          receive one of the N_TYPE_XXX via \p pValue parameter.
 *          \p hMatcher can be #NULL in this case.
 * \sa  HFccMatcher | FccmSetParameter
 *
 * Retrieves value of the specified parameter of the specified matcher
 * object.
 */
NResult N_API FccmGetParameter(HFccMatcher hMatcher, NUInt parameterId, void * pValue);

#else

#ifndef N_NO_ANSI_FUNC
NResult N_API FccmGetParameterA(HFccMatcher hMatcher, NUInt parameterId, void * pValue);
#endif
#ifndef N_NO_UNICODE
NResult N_API FccmGetParameterW(HFccMatcher hMatcher, NUInt parameterId, void * pValue);
#endif
#define FccmGetParameter N_FUNC_AW(FccmGetParameter)

#endif

#ifdef N_DOCUMENTATION
/**
 * \brief  Sets value of the parameter.
 * \param[in]  hMatcher     Handle to the matcher object. Can be #NULL if
 *                          setting static parameter value.
 * \param[in]  parameterId  Identifier of the parameter to set.
 * \param[in]  pValue       Pointer to the parameter value to set.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT</td>
 *                <td>\p pValue points to invalid value.</td></tr>
 *            <tr><td rowspan="2">#N_E_ARGUMENT_NULL</td>
 *                <td>\p parameterId specifies a non-static parameter and
 *                \p hMatcher is #NULL</td></tr>
 *            <tr><td>\p parameterId specifies a non-#N_TYPE_STRING type
 *                parameter and \p pValue is #NULL.</td></tr>
 *            <tr><td>#N_E_ARGUMENT_OUT_OF_RANGE</td>
 *                <td>\p pValue points to a value that is out of range.</td>
 *                </tr>
 *            <tr><td>#N_E_INVALID_OPERATION</td>
 *                <td>\p hMatcher is not #NULL and identification process is
 *                started.</td></tr>
 *            <tr><td>#N_E_PARAMETER</td>
 *                <td>\p parameterId is invalid.</td></tr>
 *            <tr><td>#N_E_PARAMETER_READ_ONLY</td>
 *                <td>\p parameterId specifies a read-only parameter.</td></tr>
 *          </table>
 * \remark  The following values can be used for \p parameterId:
 *          - #FCCMP_MATCHING_THRESHOLD
 *          - #FCCMP_NAME
 *          - #FCCMP_VERSION_HIGH
 *          - #FCCMP_VERSION_LOW
 *          - #FCCMP_COPYRIGHT
 * \sa  HFccMatcher | FccmGetParameter
 *
 * Sets value of the specified parameter of the specified matcher object.
 */
NResult N_API FccmSetParameter(HFccMatcher hMatcher, NUInt parameterId, const void * pValue);

#else

#ifndef N_NO_ANSI_FUNC
NResult N_API FccmSetParameterA(HFccMatcher hMatcher, NUInt parameterId, const void * pValue);
#endif
#ifndef N_NO_UNICODE
NResult N_API FccmSetParameterW(HFccMatcher hMatcher, NUInt parameterId, const void * pValue);
#endif
#define FccmSetParameter N_FUNC_AW(FccmSetParameter)

#endif

/**
 * \brief  Copies parameter values from one matcher object to another.
 * \param[in]  hDstMatcher  Handle to the destination matcher object.
 * \param[in]  hSrcMatcher  Handle to the source matcher object.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hDstMatcher or \p hSrcMatcher is #NULL.</td></tr>
 *          </table>
 * \sa  HFccMatcher
 *
 * Copies parameters values from one matcher object to another.
 */
NResult N_API FccmCopyParameters(HFccMatcher hDstMatcher, HFccMatcher hSrcMatcher);

/**
 * \brief  Verifies two templates.
 * \param[in]  hMatcher       Handle to the matcher object.
 * \param[in]  pTemplate1     Pointer to the memory buffer containing the first
 *                            facial features template.
 * \param[in]  template1Size  Size of the first facial features template.
 * \param[in]  pTemplate2     Pointer to teh memory buffer containing the
 *                            second facial features template.
 * \param[in]  template2Size  Size of the second facial features template.
 * \param[out] pScore         Pointer to #NDouble that receives similarity
 *                            score.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hMatcher, \p pTemplate1 or \p pTemplate2 is #NULL.
 *                </td></tr>
 *            <tr><td>#N_E_END_OF_STREAM</td>
 *                <td>\p pTemplate1 or \p pTemplate2 are too short buffer.</td>
 *                </tr>
 *            <tr><td>#N_E_FORMAT</td>
 *                <td>\p pTemplate1 or \p pTemplate2 are of incorrect format.
 *                </td></tr>
 *            <tr><td>#N_E_INVALID_OPERATION</td>
 *                <td>Identification process is already started.</td></tr>
 *          </table>
 * \remark  Value received via \p pScore is zero if similarity is less than a
 *          \ref FCCMP_MATCHING_THRESHOLD "matching threshold", i.e. templates
 *          do not match, and is greater than or equal to
 *          \ref FCCMP_MATCHING_THRESHOLD "matching threshold" otherwise.
 * \sa  HFccMatcher | FccmSetParameter | #FCCMP_MATCHING_THRESHOLD |
 *      FccmIdentifyStart | FccmIdentifyNext | FccmIdentifyEnd
 *
 * Verifies two facial features templates and returns their similarity score.
 */
NResult N_API FccmVerify(HFccMatcher hMatcher,
	const void *pTemplate1, NSizeType template1Size, const void *pTemplate2, NSizeType template2Size,
	NDouble *pScore);

/**
 * \brief  Initializes identification procedure.
 * \param[in]  hMatcher      Handle to the matcher object.
 * \param[in]  pTemplate     Pointer to the memory buffer containing probe
 *                           facial features template.
 * \param[in]  templateSize  Size of the facial features template.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hMatcher or \p pTemplate is #NULL.</td></tr>
 *            <tr><td>#N_E_END_OF_STREAM</td>
 *                <td>\p pTemplate is too short buffer.</td></tr>
 *            <tr><td>#N_E_FORMAT</td>
 *                <td>\p pTemplate is of incorrect format.</td></tr>
 *            <tr><td>#N_E_INVALID_OPERATION</td>
 *                <td>Identification process is already started.</td></tr>
 *          </table>
 * \sa  HFccMatcher | FccmIdentifyNext | FccmIdentifyEnd
 *
 * Initializes identification procedure with a probe template.
 */
NResult N_API FccmIdentifyStart(HFccMatcher hMatcher,
	const void *pTemplate, NSizeType templateSize);
/**
 * \brief  Compares a probe template with one of the gallery templates.
 * \param[in]  hMatcher      Handle to the matcher object.
 * \param[in]  pTemplate     Pointer to the memory buffer containing facial
 *                           features template from gallery.
 * \param[in]  templateSize  Size of the facial features template.
 * \param[out] pScore        Pointer to #NDouble that receives similarity
 *                           score.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hMatcher or \p pTemplate is #NULL.</td></tr>
 *            <tr><td>#N_E_END_OF_STREAM</td>
 *                <td>\p pTemplate is too short buffer.</td></tr>
 *            <tr><td>#N_E_FORMAT</td>
 *                <td>\p pTemplate is of incorrect format.</td></tr>
 *            <tr><td>#N_E_INVALID_OPERATION</td>
 *                <td>Identification process is not started.</td></tr>
 *          </table>
 * \remark  Value received via \p pScore is zero if similarity is less than a
 *          \ref FCCMP_MATCHING_THRESHOLD "matching threshold", i.e. templates
 *          do not match, and is greater than or equal to
 *          \ref FCCMP_MATCHING_THRESHOLD "matching threshold" otherwise.
 * \sa  HFccMatcher | FccmSetParameter | #FCCMP_MATCHING_THRESHOLD |
 *      FccmIdentifyStart | FccmIdentifyEnd
 *
 * Compares a probe template with one of the gallery templates.
 */
NResult N_API FccmIdentifyNext(HFccMatcher hMatcher,
	const void *pTemplate, NSizeType templateSize,
	NDouble *pScore);
/**
 * \brief  Finalizes identification procedure.
 * \param[in]  hMatcher  Handle to the matcher object.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hMatcher is #NULL.</td></tr>
 *            <tr><td>#N_E_INVALID_OPERATION</td>
 *                <td>Identification process is not started.</td></tr>
 *          </table>
 * \sa  HFccMatcher | FccmIdentifyStart | FccmIdentifyNext
 *
 * Finalizes a started identification procedure.
 */
NResult N_API FccmIdentifyEnd(HFccMatcher hMatcher);

#ifdef N_CPP
}
#endif

#endif // !FCC_MATCHER_H_INCLUDED
